package edu.nju.controller;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import edu.nju.entities.Bug;
import edu.nju.entities.BugHistory;
import edu.nju.service.ReportService;
import edu.nju.util.TimeUtil;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import edu.nju.service.AnalyzeService;
import edu.nju.service.HistoryService;
import edu.nju.service.RecommendService;

/**
 * 获取历史记录 /history
 */
@Controller
@RequestMapping(value = "/history")
@CrossOrigin(origins = "*", maxAge = 3600, allowCredentials = "true")
public class HistoryController {
	
	@Autowired
	HistoryService hisservice;
	
	@Autowired
	AnalyzeService aservice;
	
	@Autowired
	RecommendService recservice;

	@Autowired
	ReportService reportService;
	
	//获取指定节点的历史信息
	@RequestMapping(value = "/getHistory")
	@ResponseBody
	public void getHistory(String id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			out.print(new JSONObject(hisservice.getHistory(id)));
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//获取所有根节点
	@RequestMapping(value = "/getRoots")
	@ResponseBody
	public void getRoots(String case_take_id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> list = hisservice.getRoots(case_take_id);
			result.put("Count", list.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 教师所有形成树状结构的bug根节点
	 * @param case_take_id
	 * @param start 从第几个开始
	 * @param count 显示几个
	 * @param page 页面信息  eg:首页  or  首页——众测广场 or  首页——众测广场--任务
	 * @param session
	 * @param response
	 */
	@RequestMapping(value = "/getTrees")
	@ResponseBody
	public void getTrees(String case_take_id, String start, String count, String page, HttpSession session, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = hisservice.getTreeRoots(case_take_id);
			hisservice.pageFilter(all, page);
			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));
			List<List<String>> list = new ArrayList<List<String>>();
			for(String id: ids) {
				list.add(hisservice.getDetail(id));
			}
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

//		try {
//			PrintWriter out = response.getWriter();
//			JSONObject result = new JSONObject();
//			List<String> all = hisservice.getTreeRoots(case_take_id);
//			hisservice.pageFilter(all, page);
//			hisservice.hotSortTree(all);
//			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));
//			List<List<String>> list = new ArrayList<List<String>>();
//			for(String id: ids) {
//				list.add(hisservice.getDetail(id));
//			}
//			result.put("Count", all.size());
//			result.put("TreeRoot", new JSONArray(list));
//			out.print(result);
//			out.flush();
//			out.close();
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
	}

	/**
	 * 学生获取所有形成树状结构的bug根节点
	 * @param case_take_id
	 * @param start 从第几个开始
	 * @param count 显示几个
	 * @param page 页面信息  eg:首页  or  首页——众测广场 or  首页——众测广场--任务
	 * @param session
	 * @param response
	 */
	@GetMapping(value = "/getTreesStu")
	@ResponseBody
	public void getTreesStu(String case_take_id, String start, String count, String page, HttpSession session, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();

			List<String> all = hisservice.getTreeRoots(case_take_id);

			hisservice.pageFilter(all, page);

			hisservice.hotSortTree(all);

			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));
			List<List<String>> list = new ArrayList<List<String>>();
			for(String id: ids) {
				list.add(hisservice.getDetail(id));
			}
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 教师获取所有单个节点的数据 /getSingle
	 * @param case_take_id
	 * @param start 从第几个开始
	 * @param count 显示几个
	 * @param page 页面信息  eg:首页  or  首页——众测广场 or  首页——众测广场--任务
	 * @param session
	 * @param response
	 */
	@RequestMapping(value = "/getSingle")
	@ResponseBody
	public void getSingle(String case_take_id, String start, String count, String page, HttpSession session, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = new ArrayList<String>();
			//三级页面筛选，单个节点筛选(没有parent，也没有children)
			for(String id : hisservice.getBugIdListByPage(case_take_id,page,start,count)) {
				BugHistory bugHistory=hisservice.getHistory(id);
				if(bugHistory==null || bugHistory.getChildren().size() == 0&&bugHistory.getParent().equals("null")) {all.add(id);}
			}
			//根据三级页面筛选
			//分页
			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));

			List<String> invalid = hisservice.getInvalid(ids);
			for(String id: invalid) {
				if(ids.contains(id)) {ids.remove(id);}
			}
			List<List<String>> list = new ArrayList<List<String>>();
			for(String id : ids) {
				List<String> temp = new ArrayList<String>();
				temp.add(id);
				int score = aservice.getGrade(id);
				if(score != -1) {temp.add("true");}
				else {temp.add("false");}
				temp.add(recservice.getTitle(id));
				list.add(temp);
			}
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		try {
//			PrintWriter out = response.getWriter();
//			JSONObject result = new JSONObject();
//			List<String> all = new ArrayList<String>();
//			//三级页面筛选，单个节点筛选(没有parent，也没有children)
//			for(String id : hisservice.getBugIdListByPage(case_take_id,page,start,count)) {
//				BugHistory bugHistory=hisservice.getHistory(id);
//				if(bugHistory.getChildren().size() == 0&&bugHistory.getParent().equals("null")) {all.add(id);}
//			}
//			//根据热度排序，热度低的排在前面
//			hisservice.hotSortSingle(all);
//			//根据三级页面筛选
//			//分页
//			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));
//
//			List<String> invalid = hisservice.getInvalid(ids);
//			for(String id: invalid) {
//				if(ids.contains(id)) {ids.remove(id);}
//			}
//			//判断是否已被评分
//			List<List<String>> list = new ArrayList<List<String>>();
//			for(String id : ids) {
//				List<String> temp = new ArrayList<String>();
//				temp.add(id);
//				int score = aservice.getGrade(id);
//				if(score != -1) {temp.add("true");}
//				else {temp.add("false");}
//				temp.add(recservice.getTitle(id));
//				list.add(temp);
//			}
//			result.put("Count", all.size());
//			result.put("TreeRoot", new JSONArray(list));
//			out.print(result);
//			out.flush();
//			out.close();
//		} catch (IOException e) {
//			e.printStackTrace();
//		}
	}
	/**
	 * 所有单个节点的数据
	 * @param case_take_id
	 * @param start 从第几个开始
	 * @param count 显示几个
	 * @param page 页面信息  eg:首页  or  首页——众测广场 or  首页——众测广场--任务
	 * @param condition 是否评分 null为全部 false 未评分 true已评分
	 * @param roleId 默认为0学生 1为教师
	 * @param session
	 * @param response
	 */
	@RequestMapping(value = "/getSingleData")
	@ResponseBody
	public void getSingles(String case_take_id, String start, String count, String page,String condition,int roleId, HttpSession session, HttpServletResponse response) {
		try {
			String bugCondition="";
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = new ArrayList<String>();
			//三级页面筛选，单个节点筛选(没有parent，也没有children)
			for(String id : hisservice.getBugIdListByPage(case_take_id,page,start,count)) {
				BugHistory bugHistory=hisservice.getHistory(id);
				if(bugHistory.getChildren().size() == 0&&bugHistory.getParent().equals("null")) {
					if(condition.equals("null")){
						all.add(id);
					}else{
						int score = aservice.getGrade(id);
						if(score != -1) {
							bugCondition="true";
						}
						else {bugCondition="false";}
						if(bugCondition.equals(condition)) {
							all.add(id);
						}
					}
				}
			}

			//学生视图根据热度排序，热度低的排在前面
			// 默认为0学生 1为教师
			if(roleId == 0){
				hisservice.hotSortSingle(all);
			}

			//根据三级页面筛选
			//分页
			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));

			List<String> invalid = hisservice.getInvalid(ids);
			for(String id: invalid) {
				if(ids.contains(id)) {ids.remove(id);}
			}

			List<List<String>> list = new ArrayList<List<String>>();
			for(String id : ids) {
				List<String> temp = new ArrayList<String>();
				temp.add(id);
				int score = aservice.getGrade(id);
				if(score != -1) {temp.add("true");}
				else {temp.add("false");}
				temp.add(recservice.getTitle(id));
				list.add(temp);
			}
			//所有符合条件的，而非仅仅是展示的
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 所有形成树状结构的bug根节点
	 * @param case_take_id
	 * @param start 从第几个开始
	 * @param count 显示几个
	 * @param page 页面信息  eg:首页  or  首页——众测广场 or  首页——众测广场--任务
	 * @param roleId 默认为0学生 1为教师
	 * @param session
	 * @param response
	 */
	@RequestMapping(value = "/getTreeData")
	@ResponseBody
	public void getTree(String case_take_id, String start, String count, String page,int roleId, HttpSession session, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = hisservice.getTreeRoots(case_take_id);
			hisservice.pageFilter(all, page);
			if(roleId == 0){
				hisservice.hotSortTree(all);
			}
			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));
			List<List<String>> list = new ArrayList<List<String>>();
			for(String id: ids) {
				list.add(hisservice.getDetail(id));
			}
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	//获取所有单个节点的数据
	@RequestMapping(value = "/getAll")
	@ResponseBody
	public void getAll(String case_take_id, String start, String count, String page, HttpSession session, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = hisservice.getBugIdListByPage(case_take_id,page,start,count).stream().collect(Collectors.toList());
			//分页
			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));

			List<String> invalid = hisservice.getInvalid(ids);
			for(String id: invalid) {
				if(ids.contains(id)) {ids.remove(id);}
			}
			List<List<String>> list = new ArrayList<List<String>>();
			for(String id : ids) {
				List<String> temp = new ArrayList<String>();
				temp.add(id);
				int score = aservice.getGrade(id);
				if(score != -1) {temp.add("true");}
				else {temp.add("false");}
				Bug bug = reportService.findBugById(id);
				temp.add(bug.getTitle());
				temp.add(TimeUtil.timeStamp2Date(bug.getCreate_time_millis()));
				list.add(temp);
			}
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 学生获取所有单个节点的数据 /getSingleStu
	 * @param case_take_id
	 * @param start 从第几个开始
	 * @param count 显示几个
	 * @param page 页面信息  eg:首页  or  首页——众测广场 or  首页——众测广场--任务
	 * @param session
	 * @param response
	 */
	@GetMapping(value = "/getSingleStu")
	@ResponseBody
	public void getSingleStu(String case_take_id, String start, String count, String page, HttpSession session, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = new ArrayList<String>();
//			//获取所有root
//			for(String id : hisservice.getRoots(case_take_id)) {
//				if(hisservice.getHistory(id).getChildren().size() == 0) {all.add(id);}
//			}
//			//根据三级页面筛选
//			hisservice.pageFilter(all, page);
//
//			//根据热度排序
//			hisservice.hotSortSingle(all);
//
//			//分页
//			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));


			//三级页面筛选，单个节点筛选(没有parent，也没有children)
			for(String id : hisservice.getBugIdListByPage(case_take_id,page,start,count)) {
				BugHistory bugHistory=hisservice.getHistory(id);
				if(bugHistory.getChildren().size() == 0&&bugHistory.getParent().equals("null")) {all.add(id);}
			}
			//根据热度排序，热度低的排在前面
			hisservice.hotSortSingle(all);

			//根据三级页面筛选
			//分页
			List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));

			List<String> invalid = hisservice.getInvalid(ids);
			for(String id: invalid) {
				if(ids.contains(id)) {ids.remove(id);}
			}
			//判断是否已被评分
			List<List<String>> list = new ArrayList<List<String>>();
			for(String id : ids) {
				List<String> temp = new ArrayList<String>();
				temp.add(id);
				int score = aservice.getGrade(id);
				if(score != -1) {temp.add("true");}
				else {temp.add("false");}
				temp.add(recservice.getTitle(id));
				list.add(temp);
			}
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 获取指定bug的所有路径 /getPath
	 * @param id bugId
	 * @param response
	 */
	@GetMapping(value = "/getPath")
	@ResponseBody
	public void getPath(String id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<List<String>> lists = hisservice.getDepth(id);
			Set<String> filter = hisservice.filter(lists);
			result.put("path", new JSONArray(lists));
			result.put("invalid", new JSONArray(hisservice.getInvalid(filter)));
			List<String> ids = new ArrayList<String>(filter);
			result.put("score", new JSONArray(aservice.getScores(ids)));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	@RequestMapping(value = "/fresh")
	@ResponseBody
	public void fresh(HttpSession session, HttpServletResponse response) {
		if(session.getAttribute("trees") != null) {session.removeAttribute("trees");}
		if(session.getAttribute("single") != null) {session.removeAttribute("single");}
	}

	/**
	 * 教师通过是否评分筛选接口 /getSingleByCondition
	 * @param case_take_id
	 * @param start
	 * @param count
	 * @param page
	 * @param condition 是否评分
	 * @param session
	 * @param response
	 */
	@RequestMapping(value = "/getSingleByCondition")
	@ResponseBody
	public void getSingleByCondition(String case_take_id, String start, String count, String page,String condition, HttpSession session, HttpServletResponse response) {
		try {
			String bugCondition="";
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<String> all = new ArrayList<String>();
            for(String id : hisservice.getBugIdListByPage(case_take_id,page,start,count)) {
                BugHistory bugHistory=hisservice.getHistory(id);
                if(bugHistory.getChildren().size() == 0&&bugHistory.getParent().equals("null")) {
                    int score = aservice.getGrade(id);
					if(score != -1) {
						bugCondition="true";
					}
					else {bugCondition="false";}
					if(bugCondition.equals(condition)) {
                        all.add(id);
                    }
                }
            }

            hisservice.hotSortSingle(all);

            List<String> ids = all.subList(Integer.parseInt(start), Math.min(all.size(), Integer.parseInt(start) + Integer.parseInt(count)));

            //根据热度排序，热度低的排在前面

			List<String> invalid = hisservice.getInvalid(ids);
			for(String id: invalid) {
				if(ids.contains(id)) {ids.remove(id);}
			}

			List<List<String>> list = new ArrayList<List<String>>();
			for(String id : ids) {
				List<String> temp = new ArrayList<String>();
				temp.add(id);
				temp.add(condition);
				temp.add(recservice.getTitle(id));
				list.add(temp);
			}
			//所有符合条件的，而非仅仅是展示的
			result.put("Count", all.size());
			result.put("TreeRoot", new JSONArray(list));
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@RequestMapping(value = "/test")
	@ResponseBody
	public void getBugByPage(String case_take_id,String start, String count, String page, HttpSession session, HttpServletResponse response){
		List<String> bugList=hisservice.getBugIdListByPage(case_take_id,page,start,count);
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			result.put("Count",bugList.size());
			result.put("TreeRoot", new JSONArray(bugList));
			out.print(result);
			out.flush();
			out.close();
		}catch (IOException e){
			e.printStackTrace();
		}
	}
}
