package edu.nju.controller;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletResponse;

import edu.nju.service.UploadService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import edu.nju.service.CTBService;
import edu.nju.service.SaveService;
import org.springframework.web.multipart.MultipartFile;

/**
 * 上传 /upload
 */
@Controller
@RequestMapping(value = "/upload")
@CrossOrigin(origins = "*", maxAge = 3600, allowCredentials = "true")
public class UploadController {
	
	@Autowired
	SaveService saveservice;
	
	@Autowired
	CTBService ctbservice;

//	@Autowired
//	BlockChainAspect blockChainAspect;

	@Autowired
	UploadService uploadService;

	/**
	 * 上传新的Bug报告    /submit  返回具体bug报告的id
	 * @param useCase   用例id
	 * @param case_take_id  案例-任务id
	 * @param bug_category  bug分类
	 * @param description   写的描述信息
	 * @param img_url   图片路径，通过逗号分隔
	 * @param severity  严重等级
	 * @param recurrent 复现程度
	 * @param title     填写的bug标题
	 * @param report_id 报告id
	 * @param parent    默认为null 父报告
	 * @param page  页面信息  一级页面-二级页面-三级页面
	 * @param case_id   案例id
	 * @param worker_id 人员id
	 * @param response
	 */
	@RequestMapping(value = "/submit", method = RequestMethod.POST)
	@ResponseBody
	public void submit(String useCase, String case_take_id, String bug_category, String description, String img_url, String severity, String recurrent, String title, String report_id, String parent, String page, String case_id, String worker_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		boolean flag = true;
		String id = saveservice.save(case_take_id, bug_category, description, img_url, severity, recurrent, title, report_id, parent, page, useCase, case_id);
//		System.out.println(useCase);
		if(!useCase.equals("null") && !id.equals("")) {flag = ctbservice.save(useCase, id, case_take_id, report_id);}
//		boolean uploadBlockResult=blockChainAspect.uploadTestReportInfoToBlockChain(id);
		if(flag && !id.equals("")) {
			result.put("status", "200");
			result.put("id", id);
		}
		else {result.put("status", "500");}
		try {
			PrintWriter out = response.getWriter();
			saveservice.saveStu(report_id, worker_id);
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//对已有报告进行修改
//	@RequestMapping(value = "/update", method = RequestMethod.POST)
//	@ResponseBody
//	public void update(String id, String case_take_id, String bug_category, String description, String img_url, String severity, String recurrent, String title, String report_id, String parent, String page, HttpServletResponse response) {
//		JSONObject result = new JSONObject();
//		if(saveservice.update(id, case_take_id, bug_category, description, img_url, Integer.parseInt(severity), Integer.parseInt(recurrent), title, report_id, parent, page)) {
//			result.put("status", "200");
//		} else {
//			result.put("status", "500");
//		}
//		try {
//			PrintWriter out = response.getWriter();
//			out.print(result);
//			out.flush();
//			out.close();
//		} catch (IOException e) {
//			e.printStackTrace();
//		}
//	}

	/**
	 * 点赞接口 👍 /good
	 * @param id   被点赞的bug报告id
	 * @param report_id  本人的报告id
	 * @param response  返回200 或者 500
	 */
	@GetMapping(value = "/good")
	@ResponseBody
	public void good(String id, String report_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.confirm(id, report_id)) {
			result.put("status", "200");
		} else {
			result.put("status", "500");
		}
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 取消点赞 /cancelGood
	 * @param id   被点赞的bug报告id
	 * @param report_id  本人的报告id
	 * @param response  返回200 或者 500
	 */
	@GetMapping(value = "/cancelGood")
	@ResponseBody
	public void cancelGood(String id, String report_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.cancelGood(id, report_id)) {
			result.put("status", "200");
		} else {
			result.put("status", "500");
		}
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 点踩 /bad
	 * @param id   被点踩的bug报告id
	 * @param report_id  本人的报告id
	 * @param response  返回200 或者 500
	 */
	@GetMapping(value = "/bad")
	@ResponseBody
	public void bad(String id, String report_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.diss(id, report_id)) {
			result.put("status", "200");
		} else {
			result.put("status", "500");
		}
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 取消点踩 /bad
	 * @param id   被点踩的bug报告id
	 * @param report_id  本人的报告id
	 * @param response  返回200 或者 500
	 */
	@GetMapping(value = "/cancelBad")
	@ResponseBody
	public void cancelBad(String id, String report_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.cancelDiss(id, report_id)) {
			result.put("status", "200");
		} else {
			result.put("status", "500");
		}
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@RequestMapping(value = "/repair", method = RequestMethod.POST)
	@ResponseBody
	public void repair(String id, String page, String case_take_id, String bug_category, String description, String img_url, String severity, String recurrent, String report_id, String case_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.repair(id, page, case_take_id, bug_category, description, img_url, severity, recurrent, report_id, case_id)) {
			result.put("status", "200");
		} else {
			result.put("status", "500");
		}
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@RequestMapping(value = "/title", method = RequestMethod.POST)
	@ResponseBody
	public void saveTitle(String id, String title, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.saveTitle(id, title)) { result.put("status", "200"); }
		else { result.put("status", "500"); }
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@RequestMapping(value = "/thums")
	@ResponseBody
	public void repairThums(String case_take_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.repairThums(case_take_id)) { result.put("status", "200"); }
		else { result.put("status", "500"); }
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@RequestMapping(value = "/time")
	@ResponseBody
	public void repairTime(String case_take_id, HttpServletResponse response) {
		JSONObject result = new JSONObject();
		if(saveservice.repairTime(case_take_id)) { result.put("status", "200"); }
		else { result.put("status", "500"); }
		try {
			PrintWriter out = response.getWriter();
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 上传图片 /image
	 * @param file
	 * @param caseId
	 * @param work_id
	 * @return
	 */
	@RequestMapping(value = "/image", method = RequestMethod.POST)
	@ResponseBody
	public String uploadImage(MultipartFile file, String caseId, String work_id){
		String uploadImage = uploadService.uploadImage(file, caseId, work_id);
		System.out.println("图片上传：" + uploadImage);
		return uploadImage;
	}
}
