package edu.nju.service;

import edu.nju.dao.ConfigurationDao;
import edu.nju.model.Configuration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.ConcurrencyFailureException;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author guochao
 * @date 2021-05-11 11:39
 */
@Service
public class ConfigurationServiceImpl implements ConfigurationService {
	@Autowired
	private ConfigurationDao configurationDao;

	@Override
	public List<Configuration> getConfigurationList() {
		return configurationDao.findAll();
	}

	@Override
	public Map<String, Object> getConfigurationMap() {
		return configurationDao.findAll().stream().collect(Collectors.toMap(configuration -> configuration.getName(), configuration -> configuration.getValue()));
	}

	@Override
	public Object getConfigurationByName(String name) {
		Optional<Configuration> configurationOptional = configurationDao.findByName(name);
		if(!configurationOptional.isPresent()){
			throw new ConcurrencyFailureException(name + "配置不存在");
		}else{
			return configurationOptional.get().getValue();
		}
	}

}
